// SPDX-License-Identifier: GPL-3.0
//
// Copyright (C) 2019, Donald Dugger
//
import clock from "clock";
import document from "document";
import * as C from "../common/constants.js";
import { logger, face } from "../common/utils";
import * as utils from "../common/utils";

let cur_time = false;
let running = false;
let st_time = 0;
let end_time = 0;
let fire = 0;
let elapsed_time = null;
let laps = [];
let lap_count = 0;
laps[lap_count] = 0;

face(document);

let hrHand = document.getElementById("hours");
let minHand = document.getElementById("mins");
let secHand = document.getElementById("secs");
let tenths = document.getElementById("tenths");

function sched_intr(run, rt, intr)
{

  if (intr != 0) {
    clearTimeout(intr);
    fire = 0;
  }
  if (run || rt)
    fire = setTimeout(updateClock, 1000);
}

function ss()
{

  let icon;

  if (!running) {
    st_time = Date.now() - (end_time - st_time);
    icon = "stop_btn.png";
    if (lap_count > 0)
      lap_count--;
  } else {
    end_time = Date.now();
    icon = "start_btn.png";
    if (lap_count > 0)
      laps[lap_count++] = end_time - st_time;
  }
  running = !running;
  laps[lap_count] = end_time - st_time;
  updateClock();

  utils.btn_paint(document.getElementById("ss_btn"),
                  running ? "stop_btn.png" : "start_btn.png",
                  running ? "green" : "red");
  utils.btn_paint(document.getElementById("clear_btn"),
                  running ? "lap_btn.png" : "clear_btn.png",
                  "yellow");
}

let bg = document.getElementById("click_bg");
bg.onclick = function(evt)
{
  let num;

  cur_time = !cur_time;
  utils.show_numbers(cur_time, document);
  updateClock();
}

let ss_btn = document.getElementById("ss_btn");
ss_btn.onclick = function(evt)
{
  
  ss();
}

ss_btn.onkeypress = function(e)
{
  ss();
}

function clear()
{

  let time = Date.now();
  if (running) {
    laps[lap_count++] = time - st_time;
    laps[lap_count] = 0;
  } else {
    st_time = time;
    end_time = st_time;
    laps = [];
    lap_count = 0;
    laps[lap_count] = 0;
    updateFace(running, cur_time);
  }
}

let clear_btn = document.getElementById("clear_btn");
clear_btn.onclick = function(evt)
{
  clear();
}

clear_btn.onkeypress = function(e)
{
  clear();
}

function sh_laps()
{
  
  let disp = document.getElementById("analog_face");
  disp.style.display = "none";
  disp = document.getElementById("lap_list");
  disp.style.display = "inline";

  let list = document.getElementById("laps");
  list.delegate = {
    getTileInfo: function(index) {
      let label = "";
      let value = "";
      if (index == lap_count) {
        label = "Elapsed";
        value = utils.milli2str(laps[index], !running)
      } else {
        let prior = ((index == 0) ? 0 : laps[index - 1]);
        label = index + 1;
        value = utils.milli2str(laps[index] - prior, true)
      }
      return {
        type: "lap-pool",
        label: label,
        index: index,
        value: value
      };
    },
    configureTile: function(tile, info) {
      if (info.type == "lap-pool") {
        if (tile.getElementById("lap_no").text == "Elapsed")
          elapsed_time = null;
        if (info.index == lap_count)
          elapsed_time = tile.getElementById("lap_time");
        tile.getElementById("lap_no").text = info.label;
        tile.getElementById("lap_time").text = info.value;
      }
    }
  };

  // list.length must be set AFTER list.delegate
  list.length = lap_count + 1;
}

let list_btn = document.getElementById("list_btn");
list_btn.onclick = function(evt)
{

  elapsed_time = null;
  sh_laps();
}

let clock_btn = document.getElementById("clock_btn");
clock_btn.onclick = function(evt)
{

  let disp = document.getElementById("analog_face");
  disp.style.display = "inline";
  disp = document.getElementById("lap_list");
  disp.style.display = "none";
}

// Returns an angle (0-360) for the current hour in the day, including minutes
function hoursToAngle(hours, minutes) {
  if (cur_time) {
    let hourAngle = (360 / 12) * hours;
    let minAngle = (360 / 12 / 60) * minutes;
    return hourAngle + minAngle;
  } else
    return (360 / 60) * hours;
}

// Returns an angle (0-360) for minutes
function minutesToAngle(minutes) { return (360 / 60) * minutes }

// Returns an angle (0-360) for seconds
function secondsToAngle(seconds) { return (360 / 60) * seconds; }

// Rotate the hands every tick
function sh_hands(t, hundreds)
{
  
  tenths.text = hundreds ? utils.numPad(t.hundred, "0") : "";
  hrHand.groupTransform.rotate.angle = hoursToAngle(t.hour, t.minute);
  minHand.groupTransform.rotate.angle = minutesToAngle(t.minute);
  secHand.groupTransform.rotate.angle = secondsToAngle(t.second);

}

function updateFace(run, rt)
{

  if (rt)
    sh_hands(utils.rt_clock(), false);
  else if (run)
    sh_hands(utils.stop_time(Date.now() - st_time), false);
  else
    sh_hands(utils.stop_time(laps[lap_count]), true);
}

function updateClock()
{

  updateFace(running, cur_time);
  if (running && (elapsed_time != null)) {
    laps[lap_count] = Date.now() - st_time;
    elapsed_time.text = utils.milli2str(laps[lap_count], !running)
  }
  sched_intr(running, cur_time, fire);
}
